//go:build !runOnTask

// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0
package jobqueue

import (
	"gorm.io/gorm"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
)

func TestSaveJobWithSQLite(t *testing.T) {

	gormDB, err := gorm.Open(sqlite.Open("file::memory:?cache=shared"), &gorm.Config{})
	if err != nil {
		t.Fatalf("a error occurred while opening the database: %v", err)
	}

	manager := &Manager{database: gormDB}
	//saver := NewJobSyncer(manager)

	// Starte den DBSaver
	p := CreateAndStartJobSyncer(manager)

	ready := make(chan struct{})
	var saver *JobSyncer

	Then[*JobSyncer, *JobSyncer](p, func(value *JobSyncer) (*JobSyncer, error) {
		saver = value
		close(ready)
		return value, nil
	}, func(e error) error {
		close(ready)
		Error("Error while starting db saver", "error", err)
		return nil
	})

	<-ready

	jobID := JobID("testJob")
	job := NewJob[CounterResult](jobID, &CounterRunnable{})

	saver.AddJob(job)

	time.Sleep(100 * time.Millisecond)

	saver.Stop()

	var count int64
	gormDB.Model(&JobPersistence{}).Count(&count)
	assert.Equal(t, int64(1), count, "It should be 1 job in the database")

	// get job from database
	var jobFromDB JobPersistence
	gormDB.First(&jobFromDB, "id = ?", jobID)
	assert.Equal(t, jobID, jobFromDB.ID, "JobID should be the same")

}
